@@ -32,7 +32,7 @@ static int GetNumberOfLogicalCPUs() {
 }
 
 static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init(AvisynthToFFCPUFlags(Env->GetCPUFlags()));
+	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()),  Args[7].AsBool(false));
 
 	char ErrorMsg[1024];
 	FFMS_ErrorInfo E;
@@ -50,6 +50,7 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 	const char *AudioFile = Args[4].AsString("%sourcefile%.%trackzn%.w64");
 	int ErrorHandling = Args[5].AsInt(FFMS_IEH_IGNORE);
 	bool OverWrite = Args[6].AsBool(false);
+	const char *DemuxerStr = Args[8].AsString("default");
 
 	std::string DefaultCache(Source);
 	DefaultCache.append(".ffindex");
@@ -59,9 +60,26 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 	if (!strcmp(AudioFile, ""))
 		Env->ThrowError("FFIndex: Specifying an empty audio filename is not allowed");
 
+	int Demuxer;
+	if (!strcmp(DemuxerStr, "default"))
+		Demuxer = FFMS_SOURCE_DEFAULT;
+	else if (!strcmp(DemuxerStr, "lavf"))
+		Demuxer = FFMS_SOURCE_LAVF;
+	else if (!strcmp(DemuxerStr, "matroska"))
+		Demuxer = FFMS_SOURCE_MATROSKA;
+	else if (!strcmp(DemuxerStr, "haalimpeg"))
+		Demuxer = FFMS_SOURCE_HAALIMPEG;
+	else if (!strcmp(DemuxerStr, "haaliogg"))
+		Demuxer = FFMS_SOURCE_HAALIOGG;
+	else
+		Env->ThrowError("FFIndex: Invalid demuxer requested");
+
 	FFMS_Index *Index = NULL;
 	if (OverWrite || !(Index = FFMS_ReadIndex(CacheFile, &E))) {
-		if (!(Index = FFMS_MakeIndex(Source, IndexMask, DumpMask, FFMS_DefaultAudioFilename, (void *)AudioFile, ErrorHandling, NULL, NULL, &E)))
+		FFMS_Indexer *Indexer = FFMS_CreateIndexerWithDemuxer(Source, Demuxer, &E);
+		if (!Indexer)
+			Env->ThrowError("FFIndex: %s", E.Buffer);
+		if (!(Index = FFMS_DoIndexing(Indexer, IndexMask, DumpMask, FFMS_DefaultAudioFilename, (void *)AudioFile, ErrorHandling, NULL, NULL, &E)))
 			Env->ThrowError("FFIndex: %s", E.Buffer);
 		if (FFMS_WriteIndex(CacheFile, Index, &E)) {
 			FFMS_DestroyIndex(Index);
@@ -79,7 +97,7 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 }
 
 static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init(AvisynthToFFCPUFlags(Env->GetCPUFlags()));
+	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()), Args[15].AsBool(false));
 
 	char ErrorMsg[1024];
 	FFMS_ErrorInfo E;
@@ -126,14 +144,28 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 	if (!_stricmp(Source, Timecodes))
 		Env->ThrowError("FFVideoSource: Timecodes will overwrite the source");
 
-	std::string DefaultCache(Source);
-	DefaultCache.append(".ffindex");
-	if (!strcmp(CacheFile, ""))
-		CacheFile = DefaultCache.c_str();
-
 	FFMS_Index *Index = NULL;
-	if (Cache)
-		Index = FFMS_ReadIndex(CacheFile, &E);
+	std::string DefaultCache;
+	if (Cache) {
+		if (*CacheFile) {
+			if (!_stricmp(Source, CacheFile))
+				Env->ThrowError("FFVideoSource: Cache will overwrite the source");
+			Index = FFMS_ReadIndex(CacheFile, &E);
+		}
+		else {
+			DefaultCache = Source;
+			DefaultCache += ".ffindex";
+			CacheFile = DefaultCache.c_str();
+			Index = FFMS_ReadIndex(CacheFile, &E);
+			// Reindex if the index doesn't match the file and its name wasn't
+			// explicitly given
+			if (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS) {
+				FFMS_DestroyIndex(Index);
+				Index = 0;
+			}
+		}
+	}
+
 	if (!Index) {
 		if (!(Index = FFMS_MakeIndex(Source, 0, 0, NULL, NULL, true, NULL, NULL, &E)))
 			Env->ThrowError("FFVideoSource: %s", E.Buffer);
@@ -174,7 +206,7 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 }
 
 static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init(AvisynthToFFCPUFlags(Env->GetCPUFlags()));
+	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()), Args[5].AsBool(false));
 
 	char ErrorMsg[1024];
 	FFMS_ErrorInfo E;
@@ -193,14 +225,27 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	if (Track <= -2)
 		Env->ThrowError("FFAudioSource: No audio track selected");
 
-	std::string DefaultCache(Source);
-	DefaultCache.append(".ffindex");
-	if (!strcmp(CacheFile, ""))
-		CacheFile = DefaultCache.c_str();
-
 	FFMS_Index *Index = NULL;
-	if (Cache)
-		Index = FFMS_ReadIndex(CacheFile, &E);
+	std::string DefaultCache;
+	if (Cache) {
+		if (*CacheFile) {
+			if (!_stricmp(Source, CacheFile))
+				Env->ThrowError("FFAudioSource: Cache will overwrite the source");
+			Index = FFMS_ReadIndex(CacheFile, &E);
+		}
+		else {
+			DefaultCache = Source;
+			DefaultCache += ".ffindex";
+			CacheFile = DefaultCache.c_str();
+			Index = FFMS_ReadIndex(CacheFile, &E);
+			// Reindex if the index doesn't match the file and its name wasn't
+			// explicitly given
+			if (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS) {
+				FFMS_DestroyIndex(Index);
+				Index = 0;
+			}
+		}
+	}
 
 	// Index needs to be remade if it is an unindexed audio track
 	if (Index && Track >= 0 && Track < FFMS_GetNumTracks(Index)
@@ -256,11 +301,11 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	return Filter;
 }
 
-#ifdef WITH_LIBPOSTPROC
+#ifdef FFMS_USE_POSTPROC
 static AVSValue __cdecl CreateFFPP(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
 	return new FFPP(Args[0].AsClip(), Args[1].AsString(""), Env);
 }
-#endif // WITH_LIBPOSTPROC
+#endif // FFMS_USE_POSTPROC
 
 static AVSValue __cdecl CreateSWScale(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
 	return new SWScale(Args[0].AsClip(), Args[1].AsInt(0), Args[2].AsInt(0), Args[3].AsString("BICUBIC"), Args[4].AsString(""), Env);
@@ -276,12 +321,12 @@ static AVSValue __cdecl FFSetLogLevel(AVSValue Args, void* UserData, IScriptEnvi
 }
 
 extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* Env) {
-    Env->AddFunction("FFIndex", "[source]s[cachefile]s[indexmask]i[dumpmask]i[audiofile]s[errorhandling]i[overwrite]b", CreateFFIndex, 0);
-	Env->AddFunction("FFVideoSource", "[source]s[track]i[cache]b[cachefile]s[fpsnum]i[fpsden]i[pp]s[threads]i[timecodes]s[seekmode]i[rffmode]i[width]i[height]i[resizer]s[colorspace]s", CreateFFVideoSource, 0);
-    Env->AddFunction("FFAudioSource", "[source]s[track]i[cache]b[cachefile]s[adjustdelay]i", CreateFFAudioSource, 0);
-#ifdef WITH_LIBPOSTPROC
+    Env->AddFunction("FFIndex", "[source]s[cachefile]s[indexmask]i[dumpmask]i[audiofile]s[errorhandling]i[overwrite]b[utf8]b[demuxer]s", CreateFFIndex, 0);
+	Env->AddFunction("FFVideoSource", "[source]s[track]i[cache]b[cachefile]s[fpsnum]i[fpsden]i[pp]s[threads]i[timecodes]s[seekmode]i[rffmode]i[width]i[height]i[resizer]s[colorspace]s[utf8]b", CreateFFVideoSource, 0);
+    Env->AddFunction("FFAudioSource", "[source]s[track]i[cache]b[cachefile]s[adjustdelay]i[utf8]b", CreateFFAudioSource, 0);
+#ifdef FFMS_USE_POSTPROC
 	Env->AddFunction("FFPP", "c[pp]s", CreateFFPP, 0);
-#endif // WITH_LIBPOSTPROC
+#endif // FFMS_USE_POSTPROC
 	Env->AddFunction("SWScale", "c[width]i[height]i[resizer]s[colorspace]s", CreateSWScale, 0);
 	Env->AddFunction("FFGetLogLevel", "", FFGetLogLevel, 0);
 	Env->AddFunction("FFSetLogLevel", "i", FFSetLogLevel, 0);
