@@ -25,9 +25,10 @@ extern "C" {
 #include <libavcodec/avcodec.h>
 }
 
+#include "ffmscompat.h"
 
 
-int AvisynthToFFCPUFlags(long AvisynthFlags);
+int64_t AvisynthToFFCPUFlags(long AvisynthFlags, bool ForFFMS = true);
 PixelFormat CSNameToPIXFMT(const char *CSName, PixelFormat Default);
 int ResizerNameToSWSResizer(const char *ResizerName);
 
