@@ -25,6 +25,11 @@
 #include "audiosource.h"
 #include "indexing.h"
 
+extern "C" {
+#include <libavutil/pixdesc.h>
+}
+
+
 #ifdef FFMS_WIN_DEBUG
 #	include <windows.h>
 #endif
@@ -33,48 +38,59 @@ static bool FFmpegInited	= false;
 bool HasHaaliMPEG = false;
 bool HasHaaliOGG = false;
 int CPUFeatures = 0;
+bool GlobalUseUTF8Paths = false;
+
 
 #ifdef FFMS_WIN_DEBUG
 
 extern "C" int av_log_level;
 
 void av_log_windebug_callback(void* ptr, int level, const char* fmt, va_list vl) {
-    static int print_prefix=1;
-    static int count;
-    static char line[1024], prev[1024];
-    AVClass* avc= ptr ? *(AVClass**)ptr : NULL;
-    if(level>av_log_level)
-        return;
-#undef fprintf
-    if(print_prefix && avc) {
-        snprintf(line, sizeof(line), "[%s @ %p]", avc->item_name(ptr), ptr);
-    }else
-        line[0]=0;
+	static int print_prefix=1;
+	static int count;
+	static char line[1024] = {0}, prev[1024] = {0};
+	AVClass* avc = ptr ? *(AVClass**)ptr : NULL;
+	if(level > av_log_level)
+		return;
 
-    vsnprintf(line + strlen(line), sizeof(line) - strlen(line), fmt, vl);
+	int written = 0;
+	if(print_prefix && avc) {
+		written = snprintf(line, sizeof(line), "[%s @ %p]", avc->item_name(ptr), ptr);
+	}
 
-    print_prefix= line[strlen(line)-1] == '\n';
-    if(print_prefix && !strcmp(line, prev)){
-        count++;
-        return;
-    }
-    if(count>0){
+	written += vsnprintf(line + written, sizeof(line) - written, fmt, vl);
+
+	print_prefix = line[written-1] == '\n';
+	line[sizeof(line) - 1] = 0;
+	if(print_prefix && !strcmp(line, prev)){
+		count++;
+		return;
+	}
+	if(count > 0){
 		std::stringstream ss;
 		ss << "    Last message repeated " << count << " times\n";
 		OutputDebugStringA(ss.str().c_str());
-        count=0;
-    }
+		count = 0;
+	}
 	OutputDebugStringA(line);
-    strcpy(prev, line);
+	strcpy(prev, line);
 }
 
 #endif
 
-FFMS_API(void) FFMS_Init(int CPUFeatures) {
+FFMS_API(void) FFMS_Init(int CPUFeatures, int UseUTF8Paths) {
 	if (!FFmpegInited) {
 		av_register_all();
-#if defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS) 
-		ffms_patch_lavf_file_open();
+#ifdef _WIN32
+		if (UseUTF8Paths) {
+			ffms_patch_lavf_file_open();
+			GlobalUseUTF8Paths = true;
+		}
+		else {
+			GlobalUseUTF8Paths = false;
+		}
+#else
+		GlobalUseUTF8Paths = false;
 #endif
 #ifdef FFMS_WIN_DEBUG
 		av_log_set_callback(av_log_windebug_callback);
@@ -130,21 +146,21 @@ FFMS_API(FFMS_VideoSource *) FFMS_CreateVideoSource(const char *SourceFile, int
 	}
 }
 
-FFMS_API(FFMS_AudioSource *) FFMS_CreateAudioSource(const char *SourceFile, int Track, FFMS_Index *Index, FFMS_ErrorInfo *ErrorInfo) {
+FFMS_API(FFMS_AudioSource *) FFMS_CreateAudioSource(const char *SourceFile, int Track, FFMS_Index *Index, int DelayMode, FFMS_ErrorInfo *ErrorInfo) {
 	try {
 		switch (Index->Decoder) {
 			case FFMS_SOURCE_LAVF:
-				return new FFLAVFAudio(SourceFile, Track, Index);
+				return new FFLAVFAudio(SourceFile, Track, *Index, DelayMode);
 			case FFMS_SOURCE_MATROSKA:
-				return new FFMatroskaAudio(SourceFile, Track, Index);
+				return new FFMatroskaAudio(SourceFile, Track, *Index, DelayMode);
 #ifdef HAALISOURCE
 			case FFMS_SOURCE_HAALIMPEG:
 				if (HasHaaliMPEG)
-					return new FFHaaliAudio(SourceFile, Track, Index, FFMS_SOURCE_HAALIMPEG);
+					return new FFHaaliAudio(SourceFile, Track, *Index, FFMS_SOURCE_HAALIMPEG, DelayMode);
 				throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_NOT_AVAILABLE, "Haali MPEG/TS source unavailable");
 			case FFMS_SOURCE_HAALIOGG:
 				if (HasHaaliOGG)
-					return new FFHaaliAudio(SourceFile, Track, Index, FFMS_SOURCE_HAALIOGG);
+					return new FFHaaliAudio(SourceFile, Track, *Index, FFMS_SOURCE_HAALIOGG, DelayMode);
 				throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_NOT_AVAILABLE, "Haali OGG/OGM source unavailable");
 #endif
 			default: 
@@ -362,9 +378,13 @@ FFMS_API(int) FFMS_DefaultAudioFilename(const char *SourceFile, int Track, const
 }
 
 FFMS_API(FFMS_Indexer *) FFMS_CreateIndexer(const char *SourceFile, FFMS_ErrorInfo *ErrorInfo) {
+	return FFMS_CreateIndexerWithDemuxer(SourceFile, FFMS_SOURCE_DEFAULT, ErrorInfo);
+}
+
+FFMS_API(FFMS_Indexer *) FFMS_CreateIndexerWithDemuxer(const char *SourceFile, int Demuxer, FFMS_ErrorInfo *ErrorInfo) {
 	ClearErrorInfo(ErrorInfo);
 	try {
-		return FFMS_Indexer::CreateIndexer(SourceFile);
+		return FFMS_Indexer::CreateIndexer(SourceFile, static_cast<FFMS_Sources>(Demuxer));
 	} catch (FFMS_Exception &e) {
 		e.CopyOut(ErrorInfo);
 		return NULL;
@@ -430,9 +450,10 @@ FFMS_API(int) FFMS_WriteIndex(const char *IndexFile, FFMS_Index *Index, FFMS_Err
 }
 
 FFMS_API(int) FFMS_GetPixFmt(const char *Name) {
-	return avcodec_get_pix_fmt(Name);
+	return av_get_pix_fmt(Name);
 }
 
+
 FFMS_API(int) FFMS_GetPresentSources() {
 	int Sources = FFMS_SOURCE_LAVF | FFMS_SOURCE_MATROSKA;
 #ifdef HAALISOURCE
@@ -451,3 +472,11 @@ FFMS_API(int) FFMS_GetEnabledSources() {
 		Sources |= FFMS_SOURCE_HAALIOGG;
 	return Sources;
 }
+
+FFMS_API(const char *) FFMS_GetFormatNameI(FFMS_Indexer *Indexer) {
+	return Indexer->GetFormatName();
+}
+
+FFMS_API(int) FFMS_GetSourceTypeI(FFMS_Indexer *Indexer) {
+	return Indexer->GetSourceType();
+}
