@@ -19,23 +19,28 @@
 //  THE SOFTWARE.
 
 #include "indexing.h"
-#include <iostream>
-#include <fstream>
+
 #include <algorithm>
+#include <fstream>
+#include <iostream>
+#include <limits>
+
 
 extern "C" {
 #include <libavutil/sha1.h>
 #include <zlib.h>
 }
 
+#undef max
+
 #define INDEXID 0x53920873
 
 extern bool HasHaaliMPEG;
 extern bool HasHaaliOGG;
 
-#ifndef WITH_LIBPOSTPROC
-unsigned postproc_version(void) { return 0; } // ugly workaround to avoid lots of ifdeffing
-#endif // WITH_LIBPOSTPROC
+#ifndef FFMS_USE_POSTPROC
+unsigned postproc_version() { return 0; } // ugly workaround to avoid lots of ifdeffing
+#endif // FFMS_USE_POSTPROC
 
 struct IndexHeader {
 	uint32_t Id;
@@ -52,17 +57,18 @@ struct IndexHeader {
 };
 
 struct TrackHeader {
-		uint32_t TT;
-		uint32_t Frames;
-		int64_t Num;
-		int64_t Den;
-		uint32_t UseDTS;
+	uint32_t TT;
+	uint32_t Frames;
+	int64_t Num;
+	int64_t Den;
+	uint32_t UseDTS;
 };
 
 
 SharedVideoContext::SharedVideoContext(bool FreeCodecContext) {
 	CodecContext = NULL;
 	Parser = NULL;
+	BitStreamFilter = NULL;
 	this->FreeCodecContext = FreeCodecContext;
 	TCC = NULL;
 }
@@ -73,12 +79,10 @@ SharedVideoContext::~SharedVideoContext() {
 		if (FreeCodecContext)
 			av_freep(&CodecContext);
 	}
-
-	if (Parser)
-		av_parser_close(Parser);
-
-	if (TCC)
-		delete TCC;
+	av_parser_close(Parser);
+	if (BitStreamFilter)
+		av_bitstream_filter_close(BitStreamFilter);
+	delete TCC;
 }
 
 SharedAudioContext::SharedAudioContext(bool FreeCodecContext) {
@@ -96,9 +100,7 @@ SharedAudioContext::~SharedAudioContext() {
 		if (FreeCodecContext)
 			av_freep(&CodecContext);
 	}
-
-	if (TCC)
-		delete TCC;
+	delete TCC;
 }
 
 TFrameInfo::TFrameInfo() {
@@ -119,22 +121,20 @@ TFrameInfo TFrameInfo::VideoFrameInfo(int64_t PTS, int RepeatPict, bool KeyFrame
 	return TFrameInfo(PTS, 0, 0, RepeatPict, KeyFrame, FilePos, FrameSize);
 }
 
-TFrameInfo TFrameInfo::AudioFrameInfo(int64_t PTS, int64_t SampleStart, unsigned int SampleCount, bool KeyFrame, int64_t FilePos, unsigned int FrameSize) {
-	return TFrameInfo(PTS, SampleStart, SampleCount, 0, KeyFrame, FilePos, FrameSize);
+TFrameInfo TFrameInfo::AudioFrameInfo(int64_t PTS, int64_t SampleStart, int64_t SampleCount, bool KeyFrame, int64_t FilePos, unsigned int FrameSize) {
+	return TFrameInfo(PTS, SampleStart, static_cast<unsigned int>(SampleCount), 0, KeyFrame, FilePos, FrameSize);
 }
 
 void FFMS_Track::WriteTimecodes(const char *TimecodeFile) {
 	ffms_fstream Timecodes(TimecodeFile, std::ios::out | std::ios::trunc);
 
-	if (!Timecodes.is_open()) {
-		std::ostringstream buf;
-		buf << "Failed to open '" << TimecodeFile << "' for writing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+	if (!Timecodes.is_open())
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Failed to open '") + TimecodeFile + "' for writing");
 
 	Timecodes << "# timecode format v2\n";
 
-	for (iterator Cur=begin(); Cur!=end(); Cur++)
+	for (iterator Cur = begin(); Cur != end(); ++Cur)
 		Timecodes << std::fixed << ((Cur->PTS * TB.Num) / (double)TB.Den) << "\n";
 }
 
@@ -145,113 +145,91 @@ int FFMS_Track::FrameFromPTS(int64_t PTS) {
 	return -1;
 }
 
-int FFMS_Track::ClosestFrameFromPTS(int64_t PTS) {
-	int Frame = 0; 
-	int64_t BestDiff = 0xFFFFFFFFFFFFFFLL; // big number
-	for (int i = 0; i < static_cast<int>(size()); i++) {
-		int64_t CurrentDiff = FFABS(at(i).PTS - PTS);
-		if (CurrentDiff < BestDiff) {
-			BestDiff = CurrentDiff;
-			Frame = i;
-		}
-	}
+static bool PTSComparison(TFrameInfo FI1, TFrameInfo FI2) {
+	return FI1.PTS < FI2.PTS;
+}
 
-	return Frame;
+int FFMS_Track::ClosestFrameFromPTS(int64_t PTS) {
+	TFrameInfo F;
+	F.PTS = PTS;
+
+	iterator Pos = std::lower_bound(begin(), end(), F, PTSComparison);
+	int Frame = std::distance(begin(), Pos);
+	if (Pos == begin() || FFABS(Pos->PTS - PTS) <= FFABS((Pos - 1)->PTS - PTS))
+		return Frame;
+	return Frame - 1;
 }
 
 int FFMS_Track::FindClosestVideoKeyFrame(int Frame) {
 	Frame = FFMIN(FFMAX(Frame, 0), static_cast<int>(size()) - 1);
-	for (int i = Frame; i > 0; i--)
-		if (at(i).KeyFrame) {
-			for (int j = i; j >= 0; j--)
-				if (at(at(j).OriginalPos).KeyFrame)
-					return j;
-		}
-	return 0;
-}
-
-int FFMS_Track::FindClosestAudioKeyFrame(int64_t Sample) {
-	for (size_t i = 0; i < size(); i++) {
-		if (at(i).SampleStart == Sample && at(i).KeyFrame)
-			return i;
-		else if (at(i).SampleStart > Sample && at(i).KeyFrame)
-			return i  - 1;
-	}
-	return size() - 1;
+	for (; Frame > 0 && !at(Frame).KeyFrame; Frame--) ;
+	for (; Frame > 0 && !at(at(Frame).OriginalPos).KeyFrame; Frame--) ;
+	return Frame;
 }
 
 FFMS_Track::FFMS_Track() {
 	this->TT = FFMS_TYPE_UNKNOWN;
-	this->TB.Num = 0; 
+	this->TB.Num = 0;
 	this->TB.Den = 0;
 	this->UseDTS = false;
 }
 
 FFMS_Track::FFMS_Track(int64_t Num, int64_t Den, FFMS_TrackType TT, bool UseDTS) {
 	this->TT = TT;
-	this->TB.Num = Num; 
+	this->TB.Num = Num;
 	this->TB.Den = Den;
 	this->UseDTS = UseDTS;
 }
 
 void FFMS_Index::CalculateFileSignature(const char *Filename, int64_t *Filesize, uint8_t Digest[20]) {
 	FILE *SFile = ffms_fopen(Filename,"rb");
+	if (!SFile)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Failed to open '") + Filename + "' for hashing");
 
-	if (SFile == NULL) {
-		std::ostringstream buf;
-		buf << "Failed to open '" << Filename << "' for hashing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
-
-	const int BlockSize = 1024*1024;
-	std::vector<uint8_t> FileBuffer(BlockSize);
+	std::vector<uint8_t> FileBuffer(1024*1024, 0);
 	std::vector<uint8_t> ctxmem(av_sha1_size);
-	AVSHA1 *ctx = (AVSHA1 *)&ctxmem[0];
+	AVSHA1 *ctx = (AVSHA1 *)(&ctxmem[0]);
 	av_sha1_init(ctx);
-	
-	memset(&FileBuffer[0], 0, BlockSize);
-	fread(&FileBuffer[0], 1, BlockSize, SFile);
-	if (ferror(SFile) && !feof(SFile)) {
-		av_sha1_final(ctx, Digest);
-		fclose(SFile);
-		std::ostringstream buf;
-		buf << "Failed to read '" << Filename << "' for hashing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
-	av_sha1_update(ctx, &FileBuffer[0], BlockSize);
 
-	fseeko(SFile, -BlockSize, SEEK_END);
-	memset(&FileBuffer[0], 0, BlockSize);
-	fread(&FileBuffer[0], 1, BlockSize, SFile);
-	if (ferror(SFile) && !feof(SFile)) {
-		av_sha1_final(ctx, Digest);
-		fclose(SFile);
-		std::ostringstream buf;
-		buf << "Failed to seek with offset " << BlockSize << " from file end in '" << Filename << "' for hashing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
-	av_sha1_update(ctx, &FileBuffer[0], BlockSize);
+	try {
+		fread(&FileBuffer[0], 1, FileBuffer.size(), SFile);
+		if (ferror(SFile) && !feof(SFile))
+			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+				std::string("Failed to read '") + Filename + "' for hashing");
 
-	fseeko(SFile, 0, SEEK_END);
-	if (ferror(SFile)) {
+		av_sha1_update(ctx, &FileBuffer[0], FileBuffer.size());
+
+		fseeko(SFile, -(int)FileBuffer.size(), SEEK_END);
+		std::fill(FileBuffer.begin(), FileBuffer.end(), 0);
+		fread(&FileBuffer[0], 1, FileBuffer.size(), SFile);
+		if (ferror(SFile) && !feof(SFile)) {
+			std::ostringstream buf;
+			buf << "Failed to seek with offset " << FileBuffer.size() << " from file end in '" << Filename << "' for hashing";
+			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
+		}
+
+		av_sha1_update(ctx, &FileBuffer[0], FileBuffer.size());
+
+		fseeko(SFile, 0, SEEK_END);
+		if (ferror(SFile))
+			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+				std::string("Failed to seek to end of '") + Filename + "' for hashing");
+
+		*Filesize = ftello(SFile);
+	}
+	catch (...) {
+		fclose(SFile);
 		av_sha1_final(ctx, Digest);
-		fclose(SFile);
-		std::ostringstream buf;
-		buf << "Failed to seek to end of '" << Filename << "' for hashing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
+		throw;
 	}
-	*Filesize = ftello(SFile);
 	fclose(SFile);
-
 	av_sha1_final(ctx, Digest);
 }
 
-static bool PTSComparison(TFrameInfo FI1, TFrameInfo FI2) {
-	return FI1.PTS < FI2.PTS;
-}
-
 void FFMS_Index::Sort() {
-	for (FFMS_Index::iterator Cur=begin(); Cur!=end(); Cur++) {
+	for (FFMS_Index::iterator Cur = begin(); Cur != end(); ++Cur) {
+		if (Cur->size() > 2 && Cur->front().PTS >= Cur->back().PTS) Cur->pop_back();
 
 		for (size_t i = 0; i < Cur->size(); i++)
 			Cur->at(i).OriginalPos = i;
@@ -304,18 +282,16 @@ static unsigned int z_def(ffms_fstream *IndexStream, z_stream *stream, void *in,
 void FFMS_Index::WriteIndex(const char *IndexFile) {
 	ffms_fstream IndexStream(IndexFile, std::ios::out | std::ios::binary | std::ios::trunc);
 
-	if (!IndexStream.is_open()) {
-		std::ostringstream buf;
-		buf << "Failed to open '" << IndexFile << "' for writing";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+	if (!IndexStream.is_open())
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Failed to open '") + IndexFile + "' for writing");
 
 	z_stream stream;
 	memset(&stream, 0, sizeof(z_stream));
 	if (deflateInit(&stream, 9) != Z_OK) {
 		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, "Failed to initialize zlib");
 	}
-	
+
 	// Write the index file header
 	IndexHeader IH;
 	IH.Id = INDEXID;
@@ -327,12 +303,11 @@ void FFMS_Index::WriteIndex(const char *IndexFile) {
 	IH.LAVCVersion = avcodec_version();
 	IH.LSWSVersion = swscale_version();
 	IH.LPPVersion = postproc_version();
-	IH.LPPVersion = 0;
 	IH.FileSize = Filesize;
 	memcpy(IH.FileSignature, Digest, sizeof(Digest));
 
 	z_def(&IndexStream, &stream, &IH, sizeof(IndexHeader), 0);
-	
+
 	for (unsigned int i = 0; i < IH.Tracks; i++) {
 		FFMS_Track &ctrack = at(i);
 		TrackHeader TH;
@@ -364,8 +339,6 @@ void FFMS_Index::WriteIndex(const char *IndexFile) {
 }
 
 static unsigned int z_inf(ffms_fstream *Index, z_stream *stream, void *in, size_t in_sz, void *out, size_t out_sz) {
-	int ret;
-
 	if (out_sz == 0 || out == 0) return 0;
 	stream->next_out = (Bytef*) out;
 	stream->avail_out = out_sz;
@@ -375,21 +348,17 @@ static unsigned int z_inf(ffms_fstream *Index, z_stream *stream, void *in, size_
 		Index->read(((char*)in) + stream->avail_in, in_sz - stream->avail_in);
 		stream->next_in = (Bytef*) in;
 		stream->avail_in += Index->gcount();
-	
-		ret = inflate(stream, Z_SYNC_FLUSH);
-		switch (ret) {
+
+		switch (inflate(stream, Z_SYNC_FLUSH)) {
 		case Z_NEED_DICT:
 			inflateEnd(stream);
 			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, "Failed to read data: Dictionary error.");
-			break;
 		case Z_DATA_ERROR:
 			inflateEnd(stream);
 			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, "Failed to read data: Data error.");
-			break;
 		case Z_MEM_ERROR:
 			inflateEnd(stream);
 			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, "Failed to read data: Memory error.");
-			break;
 		case Z_STREAM_END:
 			inflateEnd(stream);
 			return out_sz - stream->avail_out;
@@ -402,41 +371,34 @@ static unsigned int z_inf(ffms_fstream *Index, z_stream *stream, void *in, size_
 void FFMS_Index::ReadIndex(const char *IndexFile) {
 	ffms_fstream Index(IndexFile, std::ios::in | std::ios::binary);
 
-	if (!Index.is_open()) {
-		std::ostringstream buf;
-		buf << "Failed to open '" << IndexFile << "' for reading";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+	if (!Index.is_open())
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Failed to open '") + IndexFile + "' for reading");
 
 	z_stream stream;
 	memset(&stream, 0, sizeof(z_stream));
 	unsigned char in[CHUNK];
-	if (inflateInit(&stream) != Z_OK) {
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, "Failed to initialize zlib");
-	}
-	
+	if (inflateInit(&stream) != Z_OK)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			"Failed to initialize zlib");
+
 	// Read the index file header
 	IndexHeader IH;
 	z_inf(&Index, &stream,  &in, CHUNK, &IH, sizeof(IndexHeader));
-	if (IH.Id != INDEXID) {
-		std::ostringstream buf;
-		buf << "'" << IndexFile << "' is not a valid index file";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
 
-	if (IH.Version != FFMS_VERSION) {
-		std::ostringstream buf;
-		buf << "'" << IndexFile << "' is not the expected index version";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+	if (IH.Id != INDEXID)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("'") + IndexFile + "' is not a valid index file");
+
+	if (IH.Version != FFMS_VERSION)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("'") + IndexFile + "' is the expected index version");
 
 	if (IH.LAVUVersion != avutil_version() || IH.LAVFVersion != avformat_version() ||
 		IH.LAVCVersion != avcodec_version() || IH.LSWSVersion != swscale_version() ||
-		IH.LPPVersion != postproc_version()) {
-		std::ostringstream buf;
-		buf << "A different FFmpeg build was used to create '" << IndexFile << "'";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+		IH.LPPVersion != postproc_version())
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("A different FFmpeg build was used to create '") + IndexFile + "'");
 
 	if (!(IH.Decoder & FFMS_GetEnabledSources()))
 		throw FFMS_Exception(FFMS_ERROR_INDEX, FFMS_ERROR_NOT_AVAILABLE,
@@ -445,7 +407,7 @@ void FFMS_Index::ReadIndex(const char *IndexFile) {
 	Decoder = IH.Decoder;
 	Filesize = IH.FileSize;
 	memcpy(Digest, IH.FileSignature, sizeof(Digest));
-	
+
 	try {
 		for (unsigned int i = 0; i < IH.Tracks; i++) {
 			TrackHeader TH;
@@ -465,16 +427,17 @@ void FFMS_Index::ReadIndex(const char *IndexFile) {
 				ctrack[j].SampleStart = ctrack[j].SampleStart + ctrack[j - 1].SampleStart;
 			}
 		}
-
-	} catch (...) {
-		std::ostringstream buf;
-		buf << "Unknown error while reading index information in '" << IndexFile << "'";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
+	}
+	catch (FFMS_Exception const&) {
+		throw;
+	}
+	catch (...) {
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Unknown error while reading index information in '") + IndexFile + "'");
 	}
 }
 
 FFMS_Index::FFMS_Index() {
-
 }
 
 FFMS_Index::FFMS_Index(int64_t Filesize, uint8_t Digest[20]) {
@@ -499,7 +462,7 @@ void FFMS_Indexer::SetErrorHandling(int ErrorHandling) {
 }
 
 void FFMS_Indexer::SetProgressCallback(TIndexCallback IC, void *ICPrivate) {
-	this->IC = IC; 
+	this->IC = IC;
 	this->ICPrivate = ICPrivate;
 }
 
@@ -508,38 +471,67 @@ void FFMS_Indexer::SetAudioNameCallback(TAudioNameCallback ANC, void *ANCPrivate
 	this->ANCPrivate = ANCPrivate;
 }
 
-FFMS_Indexer *FFMS_Indexer::CreateIndexer(const char *Filename) {
+FFMS_Indexer *FFMS_Indexer::CreateIndexer(const char *Filename, enum FFMS_Sources Demuxer) {
 	AVFormatContext *FormatContext = NULL;
 
-	if (av_open_input_file(&FormatContext, Filename, NULL, 0, NULL) != 0) {
-		std::ostringstream buf;
-		buf << "Can't open '" << Filename << "'";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
+	if (av_open_input_file(&FormatContext, Filename, NULL, 0, NULL) != 0)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Can't open '") + Filename + "'");
 
-	// Do matroska indexing instead?
-	if (!strncmp(FormatContext->iformat->name, "matroska", 8)) {
-		av_close_input_file(FormatContext);
-		return new FFMatroskaIndexer(Filename);
-	}
+	// Demuxer was not forced, probe for the best one to use
+	if (Demuxer == FFMS_SOURCE_DEFAULT) {
+		// Do matroska indexing instead?
+		if (!strncmp(FormatContext->iformat->name, "matroska", 8)) {
+			av_close_input_file(FormatContext);
+			return new FFMatroskaIndexer(Filename);
+		}
 
 #ifdef HAALISOURCE
-	// Do haali ts indexing instead?
-	if (HasHaaliMPEG && (!strcmp(FormatContext->iformat->name, "mpeg") || !strcmp(FormatContext->iformat->name, "mpegts"))) {
-		av_close_input_file(FormatContext);
-		return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIMPEG);
-	}
+		// Do haali ts indexing instead?
+		if (HasHaaliMPEG && (!strcmp(FormatContext->iformat->name, "mpeg") || !strcmp(FormatContext->iformat->name, "mpegts"))) {
+			av_close_input_file(FormatContext);
+			return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIMPEG);
+		}
 
-	if (HasHaaliOGG && !strcmp(FormatContext->iformat->name, "ogg")) {
-		av_close_input_file(FormatContext);
-		return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIOGG);
-	}
+		if (HasHaaliOGG && !strcmp(FormatContext->iformat->name, "ogg")) {
+			av_close_input_file(FormatContext);
+			return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIOGG);
+		}
 #endif
 
-	return new FFLAVFIndexer(Filename, FormatContext);
+		return new FFLAVFIndexer(Filename, FormatContext);
+	}
+
+	// someone forced a demuxer, use it
+	if (Demuxer != FFMS_SOURCE_LAVF)
+		av_close_input_file(FormatContext);
+#if !defined(HAALISOURCE)
+	if (Demuxer == FFMS_SOURCE_HAALIOGG || Demuxer == FFMS_SOURCE_HAALIMPEG) {
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_NOT_AVAILABLE, "Your binary was not compiled with support for Haali's DirectShow parsers");
+	}
+#endif // !defined(HAALISOURCE)
+
+	switch (Demuxer) {
+		case FFMS_SOURCE_LAVF:
+			return new FFLAVFIndexer(Filename, FormatContext);
+#ifdef HAALISOURCE
+		case FFMS_SOURCE_HAALIOGG:
+			if (!HasHaaliOGG)
+				throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_NOT_AVAILABLE, "Haali's Ogg parser is not available");
+			return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIOGG);
+		case FFMS_SOURCE_HAALIMPEG:
+			if (!HasHaaliMPEG)
+				throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_NOT_AVAILABLE, "Haali's MPEG PS/TS parser is not available");
+			return new FFHaaliIndexer(Filename, FFMS_SOURCE_HAALIMPEG);
+#endif
+		case FFMS_SOURCE_MATROSKA:
+			return new FFMatroskaIndexer(Filename);
+		default:
+			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_INVALID_ARGUMENT, "Invalid demuxer requested");
+	}
 }
 
-FFMS_Indexer::FFMS_Indexer(const char *Filename) : DecodingBuffer(AVCODEC_MAX_AUDIO_FRAME_SIZE * 5) {
+FFMS_Indexer::FFMS_Indexer(const char *Filename) : DecodingBuffer(AVCODEC_MAX_AUDIO_FRAME_SIZE * 10) {
 	IndexMask = 0;
 	DumpMask = 0;
 	ErrorHandling = FFMS_IEH_CLEAR_TRACK;
@@ -557,23 +549,89 @@ FFMS_Indexer::~FFMS_Indexer() {
 
 void FFMS_Indexer::WriteAudio(SharedAudioContext &AudioContext, FFMS_Index *Index, int Track, int DBSize) {
 	// Delay writer creation until after an audio frame has been decoded. This ensures that all parameters are known when writing the headers.
-	if (DBSize > 0) {
-		if (!AudioContext.W64Writer) {
-			FFMS_AudioProperties AP;
-			FillAP(AP, AudioContext.CodecContext, (*Index)[Track]);
-			int FNSize = (*ANC)(SourceFile, Track, &AP, NULL, 0, ANCPrivate);
-			std::vector<char> WName(FNSize);
-			(*ANC)(SourceFile, Track, &AP, &WName[0], FNSize, ANCPrivate);
-			std::string WN(&WName[0]);
-			try {
-				AudioContext.W64Writer = new Wave64Writer(WN.c_str(), av_get_bits_per_sample_format(AudioContext.CodecContext->sample_fmt),
-					AudioContext.CodecContext->channels, AudioContext.CodecContext->sample_rate, (AudioContext.CodecContext->sample_fmt == SAMPLE_FMT_FLT) || (AudioContext.CodecContext->sample_fmt == SAMPLE_FMT_DBL));
-			} catch (...) {
-				throw FFMS_Exception(FFMS_ERROR_WAVE_WRITER, FFMS_ERROR_FILE_WRITE,
-					"Failed to write wave data");
-			}
+	if (DBSize <= 0) return;
+
+	if (!AudioContext.W64Writer) {
+		FFMS_AudioProperties AP;
+		FillAP(AP, AudioContext.CodecContext, (*Index)[Track]);
+		int FNSize = (*ANC)(SourceFile, Track, &AP, NULL, 0, ANCPrivate);
+		if (FNSize <= 0) {
+			DumpMask = DumpMask & ~(1 << Track);
+			return;
 		}
 
-		AudioContext.W64Writer->WriteData(&DecodingBuffer[0], DBSize);
+		std::vector<char> WName(FNSize);
+		(*ANC)(SourceFile, Track, &AP, &WName[0], FNSize, ANCPrivate);
+		std::string WN(&WName[0]);
+		try {
+			AudioContext.W64Writer =
+				new Wave64Writer(WN.c_str(),
+					av_get_bits_per_sample_fmt(AudioContext.CodecContext->sample_fmt),
+					AudioContext.CodecContext->channels,
+					AudioContext.CodecContext->sample_rate,
+					(AudioContext.CodecContext->sample_fmt == AV_SAMPLE_FMT_FLT) || (AudioContext.CodecContext->sample_fmt == AV_SAMPLE_FMT_DBL));
+		} catch (...) {
+			throw FFMS_Exception(FFMS_ERROR_WAVE_WRITER, FFMS_ERROR_FILE_WRITE,
+				"Failed to write wave data");
+		}
+	}
+
+	AudioContext.W64Writer->WriteData(&DecodingBuffer[0], DBSize);
+}
+
+int64_t FFMS_Indexer::IndexAudioPacket(int Track, AVPacket *Packet, SharedAudioContext &Context, FFMS_Index &TrackIndices) {
+	AVCodecContext *CodecContext = Context.CodecContext;
+	int64_t StartSample = Context.CurrentSample;
+	int Read = 0;
+	while (Packet->size > 0) {
+		int dbsize = AVCODEC_MAX_AUDIO_FRAME_SIZE*10;
+		int Ret = avcodec_decode_audio3(CodecContext, (int16_t *)&DecodingBuffer[0], &dbsize, Packet);
+		if (Ret < 0) {
+			if (ErrorHandling == FFMS_IEH_ABORT) {
+				throw FFMS_Exception(FFMS_ERROR_CODEC, FFMS_ERROR_DECODING, "Audio decoding error");
+			} else if (ErrorHandling == FFMS_IEH_CLEAR_TRACK) {
+				TrackIndices[Track].clear();
+				IndexMask &= ~(1 << Track);
+			} else if (ErrorHandling == FFMS_IEH_STOP_TRACK) {
+				IndexMask &= ~(1 << Track);
+			}
+			break;
+		}
+		Packet->size -= Ret;
+		Packet->data += Ret;
+		Read += Ret;
+
+		CheckAudioProperties(Track, CodecContext);
+
+		if (dbsize > 0)
+			Context.CurrentSample += (dbsize * 8) / (av_get_bits_per_sample_fmt(CodecContext->sample_fmt) * CodecContext->channels);
+
+		if (DumpMask & (1 << Track))
+			WriteAudio(Context, &TrackIndices, Track, dbsize);
+	}
+	Packet->size += Read;
+	Packet->data -= Read;
+	return Context.CurrentSample - StartSample;
+}
+
+void FFMS_Indexer::CheckAudioProperties(int Track, AVCodecContext *Context) {
+	std::map<int, FFMS_AudioProperties>::iterator it = LastAudioProperties.find(Track);
+	if (it == LastAudioProperties.end()) {
+		FFMS_AudioProperties &AP = LastAudioProperties[Track];
+		AP.SampleRate = Context->sample_rate;
+		AP.SampleFormat = Context->sample_fmt;
+		AP.Channels = Context->channels;
+	}
+	else if (it->second.SampleRate   != Context->sample_rate ||
+			 it->second.SampleFormat != Context->sample_fmt ||
+			 it->second.Channels     != Context->channels) {
+		std::ostringstream buf;
+		buf <<
+			"Audio format change detected. This is currently unsupported."
+			<< " Channels: " << it->second.Channels << " -> " << Context->channels << ";"
+			<< " Sample rate: " << it->second.SampleRate << " -> " << Context->sample_rate << ";"
+			<< " Sample format: " << GetLAVCSampleFormatName((AVSampleFormat)it->second.SampleFormat) << " -> "
+			<< GetLAVCSampleFormatName(Context->sample_fmt);
+		throw FFMS_Exception(FFMS_ERROR_UNSUPPORTED, FFMS_ERROR_DECODING, buf.str());
 	}
 }
