@@ -26,15 +26,12 @@
 #ifdef _WIN32
 #	define WIN32_LEAN_AND_MEAN
 #	include <windows.h>
-#ifdef FFMS_USE_UTF8_PATHS
 #	include <io.h>
 #	include <fcntl.h>
 extern "C" {
 #	include "libavutil/avstring.h"
 }
-#endif
-#endif
-
+#endif // _WIN32
 
 
 // Export the array but not its data type... fun...
@@ -48,9 +45,15 @@ extern const AVCodecTag ff_codec_bmp_tags[];
 extern const CodecTags ff_mkv_codec_tags[];
 extern const AVCodecTag ff_codec_movvideo_tags[];
 extern const AVCodecTag ff_codec_wav_tags[];
+
+/* if you have this, we'll assume you have a new enough libavutil too */
+#if LIBSWSCALE_VERSION_INT >= AV_VERSION_INT(0, 12, 0)
+#	include <libavutil/opt.h>
+#endif
 }
 
 extern int CPUFeatures;
+extern bool GlobalUseUTF8Paths;
 
 
 
@@ -71,7 +74,7 @@ int FFMS_Exception::CopyOut(FFMS_ErrorInfo *ErrorInfo) const {
 	if (ErrorInfo) {
 		ErrorInfo->ErrorType = _ErrorType;
 		ErrorInfo->SubType = _SubType;
-		
+
 		if (ErrorInfo->BufferSize > 0) {
 			memset(ErrorInfo->Buffer, 0, ErrorInfo->BufferSize);
 			_Message.copy(ErrorInfo->Buffer, ErrorInfo->BufferSize - 1);
@@ -99,7 +102,7 @@ TrackCompressionContext::TrackCompressionContext(MatroskaFile *MF, TrackInfo *TI
 		CompressedPrivateData		= TI->CompMethodPrivate;
 		CompressedPrivateDataSize	= TI->CompMethodPrivateSize;
 	} else {
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, 
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
 			"Can't create MKV track decompressor: unknown or unsupported compression method");
 	}
 }
@@ -109,8 +112,8 @@ TrackCompressionContext::~TrackCompressionContext() {
 		cs_Destroy(CS);
 }
 
-int GetSWSCPUFlags() {
-	int Flags = 0;
+int64_t GetSWSCPUFlags() {
+	int64_t Flags = 0;
 
 	if (CPUFeatures & FFMS_CPU_CAPS_MMX)
 		Flags |= SWS_CPU_CAPS_MMX;
@@ -122,14 +125,64 @@ int GetSWSCPUFlags() {
 		Flags |= SWS_CPU_CAPS_ALTIVEC;
 	if (CPUFeatures & FFMS_CPU_CAPS_BFIN)
 		Flags |= SWS_CPU_CAPS_BFIN;
+#ifdef SWS_CPU_CAPS_SSE2
+	if (CPUFeatures & FFMS_CPU_CAPS_SSE2)
+		Flags |= SWS_CPU_CAPS_SSE2;
+#endif
 
 	return Flags;
 }
 
+static int handle_jpeg(PixelFormat *format)
+{
+	switch (*format) {
+	case PIX_FMT_YUVJ420P: *format = PIX_FMT_YUV420P; return 1;
+	case PIX_FMT_YUVJ422P: *format = PIX_FMT_YUV422P; return 1;
+	case PIX_FMT_YUVJ444P: *format = PIX_FMT_YUV444P; return 1;
+	case PIX_FMT_YUVJ440P: *format = PIX_FMT_YUV440P; return 1;
+	default:                                          return 0;
+	}
+}
+SwsContext *GetSwsContext(int SrcW, int SrcH, PixelFormat SrcFormat, int DstW, int DstH, PixelFormat DstFormat, int64_t Flags, int ColorSpace) {
+#if LIBSWSCALE_VERSION_INT < AV_VERSION_INT(0, 12, 0)
+	return sws_getContext(SrcW, SrcH, SrcFormat, DstW, DstH, DstFormat, Flags, 0, 0, 0);
+#else
+	SwsContext *Context = sws_alloc_context();
+	if (!Context) return 0;
+
+	if (ColorSpace == -1)
+		ColorSpace = (SrcW > 1024 || SrcH >= 600) ? SWS_CS_ITU709 : SWS_CS_DEFAULT;
+
+	int SrcRange = handle_jpeg(&SrcFormat);
+	int DstRange = handle_jpeg(&DstFormat);
+
+	av_set_int(Context, "sws_flags", Flags);
+	av_set_int(Context, "srcw",       SrcW);
+	av_set_int(Context, "srch",       SrcH);
+	av_set_int(Context, "dstw",       DstW);
+	av_set_int(Context, "dsth",       DstH);
+	av_set_int(Context, "src_range",  SrcRange);
+	av_set_int(Context, "dst_range",  DstRange);
+	av_set_int(Context, "src_format", SrcFormat);
+	av_set_int(Context, "dst_format", DstFormat);
+
+	sws_setColorspaceDetails(Context, sws_getCoefficients(ColorSpace), SrcRange, sws_getCoefficients(ColorSpace), DstRange, 0, 1<<16, 1<<16);
+
+	if(sws_init_context(Context, 0, 0) < 0){
+		sws_freeContext(Context);
+		return 0;
+	}
+
+	return Context;
+#endif
+
+}
+
+
 int GetPPCPUFlags() {
 	int Flags = 0;
 
-#ifdef WITH_LIBPOSTPROC
+#ifdef FFMS_USE_POSTPROC
 // not exactly a pretty solution but it'll never get called anyway
 	if (CPUFeatures & FFMS_CPU_CAPS_MMX)
 		Flags |= PP_CPU_CAPS_MMX;
@@ -139,7 +192,7 @@ int GetPPCPUFlags() {
 		Flags |= PP_CPU_CAPS_3DNOW;
 	if (CPUFeatures & FFMS_CPU_CAPS_ALTIVEC)
 		Flags |= PP_CPU_CAPS_ALTIVEC;
-#endif // WITH_LIBPOSTPROC
+#endif // FFMS_USE_POSTPROC
 
 	return Flags;
 }
@@ -163,18 +216,28 @@ FFMS_TrackType HaaliTrackTypeToFFTrackType(int TT) {
 	}
 }
 
-const char *GetLAVCSampleFormatName(SampleFormat s) {
+const char *GetLAVCSampleFormatName(AVSampleFormat s) {
 	switch (s) {
-		case SAMPLE_FMT_U8:		return "8-bit unsigned integer";
-		case SAMPLE_FMT_S16:	return "16-bit signed integer";
-		case SAMPLE_FMT_S32:	return "32-bit signed integer";
-		case SAMPLE_FMT_FLT:	return "Single-precision floating point";
-		case SAMPLE_FMT_DBL:	return "Double-precision floating point";
+		case AV_SAMPLE_FMT_U8:	return "8-bit unsigned integer";
+		case AV_SAMPLE_FMT_S16:	return "16-bit signed integer";
+		case AV_SAMPLE_FMT_S32:	return "32-bit signed integer";
+		case AV_SAMPLE_FMT_FLT:	return "Single-precision floating point";
+		case AV_SAMPLE_FMT_DBL:	return "Double-precision floating point";
 		default:				return "Unknown";
 	}
 }
 
+template<class T> static void safe_aligned_reallocz(T *&ptr, size_t old_size, size_t new_size) {
+	void *newalloc = av_mallocz(new_size);
+	if (newalloc) {
+		memcpy(newalloc, ptr, min(old_size, new_size));
+	}
+	av_free(ptr);
+	ptr = static_cast<T*>(newalloc);
+}
+
 void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContext *TCC, MatroskaReaderContext &Context) {
+	memset(Context.Buffer, 0, Context.BufferSize); // necessary to avoid lavc hurfing a durf with some mpeg4 video streams
 	if (TCC && TCC->CS) {
 		CompressedStream *CS = TCC->CS;
 		unsigned int DecompressedFrameSize = 0;
@@ -191,17 +254,16 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContex
 
 			if (ReadBytes == 0) {
 				FrameSize = DecompressedFrameSize;
-				memset(Context.Buffer + DecompressedFrameSize, 0,
-					Context.BufferSize  + FF_INPUT_BUFFER_PADDING_SIZE - DecompressedFrameSize);
 				return;
 			}
 
-			if (Context.BufferSize < DecompressedFrameSize + ReadBytes) {
-				Context.BufferSize = DecompressedFrameSize + ReadBytes;
-				Context.Buffer = (uint8_t *)realloc(Context.Buffer, Context.BufferSize + FF_INPUT_BUFFER_PADDING_SIZE);
+			if (Context.BufferSize < DecompressedFrameSize + ReadBytes + FF_INPUT_BUFFER_PADDING_SIZE) {
+				size_t NewSize = (DecompressedFrameSize + ReadBytes) * 2;
+				safe_aligned_reallocz(Context.Buffer, Context.BufferSize, NewSize);
 				if (Context.Buffer == NULL)
 					throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_ALLOCATION_FAILED,
 					"Out of memory");
+				Context.BufferSize = NewSize;
 			}
 
 			memcpy(Context.Buffer + DecompressedFrameSize, Context.CSBuffer, ReadBytes);
@@ -215,12 +277,13 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContex
 		}
 
 		if (TCC && TCC->CompressionMethod == COMP_PREPEND) {
-			unsigned ReqBufsize = FrameSize + TCC->CompressedPrivateDataSize + 16;
+			unsigned ReqBufsize = FrameSize + TCC->CompressedPrivateDataSize + FF_INPUT_BUFFER_PADDING_SIZE;
 			if (Context.BufferSize < ReqBufsize) {
-				Context.BufferSize = FrameSize + TCC->CompressedPrivateDataSize;
-				Context.Buffer = (uint8_t *)realloc(Context.Buffer, ReqBufsize);
+				size_t NewSize = ReqBufsize * 2;
+				safe_aligned_reallocz(Context.Buffer, Context.BufferSize, NewSize);
 				if (Context.Buffer == NULL)
 					throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_ALLOCATION_FAILED, "Out of memory");
+				Context.BufferSize = NewSize;
 			}
 
 			/* // maybe faster? maybe not?
@@ -230,12 +293,13 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContex
 			// screw it, memcpy and fuck the losers who use header compression
 			memcpy(Context.Buffer, TCC->CompressedPrivateData, TCC->CompressedPrivateDataSize);
 		}
-		else if (Context.BufferSize < FrameSize) {
-			Context.BufferSize = FrameSize;
-			Context.Buffer = (uint8_t *)realloc(Context.Buffer, Context.BufferSize + 16);
+		else if (Context.BufferSize < FrameSize + FF_INPUT_BUFFER_PADDING_SIZE) {
+			size_t NewSize = FrameSize * 2;
+			safe_aligned_reallocz(Context.Buffer, Context.BufferSize, NewSize);
 			if (Context.Buffer == NULL)
 				throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_ALLOCATION_FAILED,
 					"Out of memory");
+			Context.BufferSize = NewSize;
 		}
 
 		uint8_t *TargetPtr = Context.Buffer;
@@ -244,7 +308,6 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContex
 
 		size_t ReadBytes = fread(TargetPtr, 1, FrameSize, Context.ST.fp);
 		if (ReadBytes != FrameSize) {
-			return;
 			if (ReadBytes == 0) {
 				if (feof(Context.ST.fp)) {
 					throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
@@ -274,16 +337,18 @@ void InitNullPacket(AVPacket &pkt) {
 
 void FillAP(FFMS_AudioProperties &AP, AVCodecContext *CTX, FFMS_Track &Frames) {
 	AP.SampleFormat = static_cast<FFMS_SampleFormat>(CTX->sample_fmt);
-	AP.BitsPerSample = av_get_bits_per_sample_format(CTX->sample_fmt);
-	if (CTX->sample_fmt == SAMPLE_FMT_S32 && CTX->bits_per_raw_sample)
+	AP.BitsPerSample = av_get_bits_per_sample_fmt(CTX->sample_fmt);
+	if (CTX->sample_fmt == AV_SAMPLE_FMT_S32 && CTX->bits_per_raw_sample)
 		AP.BitsPerSample = CTX->bits_per_raw_sample;
 
 	AP.Channels = CTX->channels;;
 	AP.ChannelLayout = CTX->channel_layout;
 	AP.SampleRate = CTX->sample_rate;
-	AP.NumSamples = (Frames.back()).SampleStart + (Frames.back()).SampleCount;
-	AP.FirstTime = ((Frames.front().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
-	AP.LastTime = ((Frames.back().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
+	if (Frames.size() > 0) {
+		AP.NumSamples = (Frames.back()).SampleStart + (Frames.back()).SampleCount;
+		AP.FirstTime = ((Frames.front().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
+		AP.LastTime = ((Frames.back().PTS * Frames.TB.Num) / (double)Frames.TB.Den) / 1000;
+	}
 }
 
 #ifdef HAALISOURCE
@@ -327,7 +392,7 @@ CodecID MatroskaToFFCodecID(char *Codec, void *CodecPrivate, unsigned int FourCC
 							case 16: CID = CODEC_ID_PCM_S16LE; break;
 							case 24: CID = CODEC_ID_PCM_S24LE; break;
 							case 32: CID = CODEC_ID_PCM_S32LE; break;
-						}	
+						}
 						break;
 					case CODEC_ID_PCM_S16BE:
 						switch (BitsPerSample) {
@@ -335,12 +400,12 @@ CodecID MatroskaToFFCodecID(char *Codec, void *CodecPrivate, unsigned int FourCC
 							case 16: CID = CODEC_ID_PCM_S16BE; break;
 							case 24: CID = CODEC_ID_PCM_S24BE; break;
 							case 32: CID = CODEC_ID_PCM_S32BE; break;
-						}	
+						}
 						break;
 					default:
 						break;
 				}
-				
+
 				return CID;
 			}
 	}
@@ -385,42 +450,85 @@ void InitializeCodecContextFromMatroskaTrackInfo(TrackInfo *TI, AVCodecContext *
 
 #ifdef HAALISOURCE
 
-void InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag, AVCodecContext *CodecContext) {
-	if (pBag) {
-		CComVariant pV;
+FFCodecContext InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag) {
+	CComVariant pV;
+	if (FAILED(pBag->Read(L"Type", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+		return FFCodecContext();
+
+	unsigned int TT = pV.uintVal;
+
+	FFCodecContext CodecContext(avcodec_alloc_context(), DeleteHaaliCodecContext);
+
+	unsigned int FourCC = 0;
+	if (TT == TT_VIDEO) {
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"Video.PixelWidth", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			CodecContext->coded_width = pV.uintVal;
 
 		pV.Clear();
-		if (SUCCEEDED(pBag->Read(L"Type", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4))) {
+		if (SUCCEEDED(pBag->Read(L"Video.PixelHeight", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			CodecContext->coded_height = pV.uintVal;
 
-			unsigned int TT = pV.uintVal;
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"FOURCC", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			FourCC = pV.uintVal;
 
-			if (TT == TT_VIDEO) {
-
-				pV.Clear();
-				if (SUCCEEDED(pBag->Read(L"Video.PixelWidth", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
-					CodecContext->coded_width = pV.uintVal;
-
-				pV.Clear();
-				if (SUCCEEDED(pBag->Read(L"Video.PixelHeight", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
-					CodecContext->coded_height = pV.uintVal;
-
-			} else if (TT == TT_AUDIO) {
-
-				pV.Clear();
-				if (SUCCEEDED(pBag->Read(L"Audio.SamplingFreq", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
-					CodecContext->sample_rate = pV.uintVal;
-
-				pV.Clear();
-				if (SUCCEEDED(pBag->Read(L"Audio.BitDepth", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
-					CodecContext->bits_per_coded_sample = pV.uintVal;
-
-				pV.Clear();
-				if (SUCCEEDED(pBag->Read(L"Audio.Channels", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
-					CodecContext->channels = pV.uintVal;
-
-			}
+		FFMS_BITMAPINFOHEADER bih;
+		memset(&bih, 0, sizeof bih);
+		if (FourCC) {
+			// Reconstruct the missing codec private part for VC1
+			bih.biSize = sizeof bih;
+			bih.biCompression = FourCC;
+			bih.biBitCount = 24;
+			bih.biPlanes = 1;
+			bih.biHeight = CodecContext->coded_height;
 		}
-  	}
+
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"CodecPrivate", &pV, NULL))) {
+			bih.biSize += vtSize(pV);
+			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(bih.biSize));
+			// prepend BITMAPINFOHEADER if there's anything interesting in it (i.e. we're decoding VC1)
+			if (FourCC)
+				memcpy(CodecContext->extradata, &bih, sizeof bih); 
+			vtCopy(pV, CodecContext->extradata + (FourCC ? sizeof bih : 0));
+		}
+		// use the BITMAPINFOHEADER only. not sure if this is correct or if it's ever going to be used...
+		else {
+			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(bih.biSize));
+			memcpy(CodecContext->extradata, &bih, sizeof bih);
+		}
+		CodecContext->extradata_size = bih.biSize;
+	}
+	else if (TT == TT_AUDIO) {
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"CodecPrivate", &pV, NULL))) {
+			CodecContext->extradata_size = vtSize(pV);
+			CodecContext->extradata = static_cast<uint8_t*>(av_malloc(CodecContext->extradata_size));
+			vtCopy(pV, CodecContext->extradata);
+		}
+
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"Audio.SamplingFreq", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			CodecContext->sample_rate = pV.uintVal;
+
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"Audio.BitDepth", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			CodecContext->bits_per_coded_sample = pV.uintVal;
+
+		pV.Clear();
+		if (SUCCEEDED(pBag->Read(L"Audio.Channels", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_UI4)))
+			CodecContext->channels = pV.uintVal;
+	}
+
+	pV.Clear();
+	if (SUCCEEDED(pBag->Read(L"CodecID", &pV, NULL)) && SUCCEEDED(pV.ChangeType(VT_BSTR))) {
+		char CodecStr[2048];
+		wcstombs(CodecStr, pV.bstrVal, 2000);
+
+		CodecContext->codec = avcodec_find_decoder(MatroskaToFFCodecID(CodecStr, CodecContext->extradata, FourCC, CodecContext->bits_per_coded_sample));
+	}
+	return CodecContext;
 }
 
 #endif
@@ -448,10 +556,12 @@ static wchar_t *dup_char_to_wchar(const char *s, unsigned int cp) {
 
 FILE *ffms_fopen(const char *filename, const char *mode) {
 #ifdef _WIN32
-	int codepage = CP_ACP;
-#ifdef FFMS_USE_UTF8_PATHS
-	codepage = CP_UTF8;
-#endif
+	unsigned int codepage;
+	if (GlobalUseUTF8Paths)
+		codepage = CP_UTF8;
+	else
+		codepage = CP_ACP;
+
 	FILE *ret;
 	wchar_t *filename_wide	= dup_char_to_wchar(filename, codepage);
 	wchar_t *mode_wide		= dup_char_to_wchar(mode, codepage);
@@ -470,18 +580,9 @@ FILE *ffms_fopen(const char *filename, const char *mode) {
 }
 
 size_t ffms_mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max) {
-#if defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS)
-	// try utf8 first
-	int len = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, mbstr, -1, NULL, 0);
-	if (len > 0) {
-		MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, mbstr, -1, wcstr, max);
-		return static_cast<size_t>(len);
-	}
-	// failed, use local ANSI codepage
-	else {
-		len = MultiByteToWideChar(CP_ACP, NULL, mbstr, -1, wcstr, max);
-		return static_cast<size_t>(len);
-	}
+#ifdef _WIN32
+	// this is only called by HaaliOpenFile anyway, so I think this is safe
+	return static_cast<size_t>(MultiByteToWideChar((GlobalUseUTF8Paths ? CP_UTF8 : CP_ACP), MB_ERR_INVALID_CHARS, mbstr, -1, wcstr, max));
 #else
 	return mbstowcs(wcstr, mbstr, max);
 #endif
@@ -494,10 +595,8 @@ void ffms_fstream::open(const char *filename, std::ios_base::openmode mode) {
 	// that takes a wchar_t* filename, which means you can't open unicode
 	// filenames with it on Windows. gg.
 #if defined(_WIN32) && !defined(__MINGW32__)
-	unsigned int codepage = CP_ACP;
-#if defined(FFMS_USE_UTF8_PATHS)
-	codepage = CP_UTF8;
-#endif /* defined(FFMS_USE_UTF8_PATHS) */
+	unsigned int codepage = GlobalUseUTF8Paths ? CP_UTF8 : CP_ACP;
+
 	wchar_t *filename_wide = dup_char_to_wchar(filename, codepage);
 	if (filename_wide)
 		std::fstream::open(filename_wide, mode);
@@ -505,9 +604,9 @@ void ffms_fstream::open(const char *filename, std::ios_base::openmode mode) {
 		std::fstream::open(filename, mode);
 
 	free(filename_wide);
-#else /* defined(_WIN32) && !defined(__MINGW32__) */
+#else // defined(_WIN32) && !defined(__MINGW32__)
 	std::fstream::open(filename, mode);
-#endif /* defined(_WIN32) && !defined(__MINGW32__) */
+#endif // defined(_WIN32) && !defined(__MINGW32__)
 }
 
 ffms_fstream::ffms_fstream(const char *filename, std::ios_base::openmode mode) {
@@ -515,12 +614,12 @@ ffms_fstream::ffms_fstream(const char *filename, std::ios_base::openmode mode) {
 }
 
 
-#if defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS)
+#ifdef _WIN32
 int ffms_wchar_open(const char *fname, int oflags, int pmode) {
     wchar_t *wfname = dup_char_to_wchar(fname, CP_UTF8);
     if (wfname) {
         int ret = _wopen(wfname, oflags, pmode);
-        av_free(wfname);
+        free(wfname);
         return ret;
     }
     return -1;
@@ -552,8 +651,7 @@ static int ffms_lavf_file_open(URLContext *h, const char *filename, int flags) {
 // Hijack lavf's file protocol handler's open function and use our own instead.
 // Hack by nielsm.
 void ffms_patch_lavf_file_open() {
-	extern URLProtocol *first_protocol;
-	URLProtocol *proto = first_protocol;
+	URLProtocol *proto = av_protocol_next(NULL);
 	while (proto != NULL) {
 		if (strcmp("file", proto->name) == 0) {
 			break;
@@ -564,7 +662,7 @@ void ffms_patch_lavf_file_open() {
 		proto->url_open = &ffms_lavf_file_open;
 	}
 }
-#endif /* defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS) */
+#endif // _WIN32
 
 // End of filename hackery.
 
@@ -614,12 +712,10 @@ CComPtr<IMMContainer> HaaliOpenFile(const char *SourceFile, enum FFMS_Sources So
 #endif
 
 void LAVFOpenFile(const char *SourceFile, AVFormatContext *&FormatContext) {
-	if (av_open_input_file(&FormatContext, SourceFile, NULL, 0, NULL) != 0) {
-		std::ostringstream buf;
-		buf << "Couldn't open '" << SourceFile << "'";
-		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
-	}
-	
+	if (av_open_input_file(&FormatContext, SourceFile, NULL, 0, NULL) != 0)
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
+			std::string("Couldn't open '") + SourceFile + "'");
+
 	if (av_find_stream_info(FormatContext) < 0) {
 		av_close_input_file(FormatContext);
 		FormatContext = NULL;
